#include <bits/stdc++.h>

using namespace std;

#define MAXN 50
#define MAXA 10
#define MAXB 5

char tiles[MAXN][MAXA][MAXA+1];
int A, B, N;

int res[MAXB][MAXB];    // store the solution, that is, the tile for each cell
bool used[MAXN];        // flag used tiles

enum side { DOWN = 0, RIGHT, UP, LEFT };

int tile2bits[MAXN][4];         // bit representation of edges
vector<int> bits2tiles[4][(1 << MAXA) - 1];     // lists of tiles compatible with each edge (perfect hash is affordable; unordered_map would timeout on 2 instances)
vector<int> all_tiles;  // utility vector to iterate over all tiles

int row2bits(int n, int i) {
    int res = 0;
    for(int j = 0; j < A; j++) res = (res << 1) | (tiles[n][i][j] - '0');
    return res;
}

int col2bits(int n, int j) {
    int res = 0;
    for(int i = 0; i < A; i++) res = (res << 1) | (tiles[n][i][j] - '0');
    return res;
}

bool search(int i = 0, int j = 0) {
    if(i == B) return true;             // all done!
    if(j == B) return search(i+1, 0);   // row i done!

    // iterate over the shortest list of "compatible" tiles
    vector<int>* next = &all_tiles;
    if(i > 0) next = &bits2tiles[UP][tile2bits[res[i-1][j]][DOWN]];
    if(j > 0 && bits2tiles[LEFT][tile2bits[res[i][j-1]][RIGHT]].size() < next->size()) next = &bits2tiles[LEFT][tile2bits[res[i][j-1]][RIGHT]];
    for(auto n : *next) {
        // skip used tiles and tiles incompatible with some edge
        if(used[n]) continue;
        if(i > 0 && tile2bits[res[i-1][j]][DOWN] != tile2bits[n][UP]) continue;
        if(j > 0 && tile2bits[res[i][j-1]][RIGHT] != tile2bits[n][LEFT]) continue;
        
        // try tile n, and in case backtrack
        used[res[i][j] = n] = true;
        if(search(i, j+1)) return true;
        used[n] = false;
    }
    
    // no way
    return false;
}

int main() {
    FILE *fr, *fw;

    fr = fopen("input.txt", "r");
    fw = fopen("output.txt", "w");
    assert(3 == fscanf(fr, "%d %d %d", &A, &B, &N));
    for(int i=0; i<N; i++)
        for(int j=0; j<A; j++)
            assert(1 == fscanf(fr, "%s", tiles[i][j]));

    // init bit representation of edges, and lists of tiles
    for(int n = 0; n < N; n++) {
        bits2tiles[DOWN][tile2bits[n][DOWN] = row2bits(n, A-1)].push_back(n);
        bits2tiles[RIGHT][tile2bits[n][RIGHT] = col2bits(n, A-1)].push_back(n);
        bits2tiles[UP][tile2bits[n][UP] = row2bits(n, 0)].push_back(n);
        bits2tiles[LEFT][tile2bits[n][LEFT] = col2bits(n, 0)].push_back(n);
        all_tiles.push_back(n);
    }
    
    // solve by backtracking
    bool ok = search();
    assert(ok);
    
    // print solution
    for(int i = 0; i < B; i++) {
        for(int j = 0; j < B; j++) fprintf(fw, "%d ", res[i][j]);
        fprintf(fw, "\n");
    }
    
    fclose(fr);
    fclose(fw);
    return 0;
}
